@@ -15,6 +15,7 @@ _fldmap = {
         'ev_time': 'ev_time',
         'ev_type': 'ev_type',
         'ev_data': 'ev_data',
+        'ev_retry': 'ev_retry',
         'ev_extra1': 'ev_extra1',
         'ev_extra2': 'ev_extra2',
         'ev_extra3': 'ev_extra3',
@@ -25,6 +26,7 @@ _fldmap = {
         'time': 'ev_time',
         'type': 'ev_type',
         'data': 'ev_data',
+        'retry': 'ev_retry',
         'extra1': 'ev_extra1',
         'extra2': 'ev_extra2',
         'extra3': 'ev_extra3',
