@@ -1,139 +1,3 @@
-create or replace function londiste.find_column_types(tbl text)
-returns text as $$
-declare
-    res      text;
-    col      record;
-    tbl_oid  oid;
-begin
-    tbl_oid := londiste.find_table_oid(tbl);
-    res := '';
-    for col in 
-        SELECT CASE WHEN k.attname IS NOT NULL THEN 'k' ELSE 'v' END AS type
-            FROM pg_attribute a LEFT JOIN (
-                SELECT k.attname FROM pg_index i, pg_attribute k
-                 WHERE i.indrelid = tbl_oid AND k.attrelid = i.indexrelid
-                   AND i.indisprimary AND k.attnum > 0 AND NOT k.attisdropped
-                ) k ON (k.attname = a.attname)
-            WHERE a.attrelid = tbl_oid AND a.attnum > 0 AND NOT a.attisdropped
-            ORDER BY a.attnum
-    loop
-        res := res || col.type;
-    end loop;
-
-    return res;
-end;
-$$ language plpgsql strict stable;
-
-
-create or replace function londiste.find_table_fkeys(i_table_name text)
-returns setof londiste.subscriber_pending_fkeys as $$
-declare
-    fkey      record;
-    tbl_oid   oid;
-begin
-    select londiste.find_table_oid(i_table_name) into tbl_oid;
-        
-    for fkey in
-        select n1.nspname || '.' || t1.relname as from_table, n2.nspname || '.' || t2.relname as to_table,
-            conname::text as fkey_name, 
-            'alter table only ' || quote_ident(n1.nspname) || '.' || quote_ident(t1.relname)
-            || ' add constraint ' || quote_ident(conname::text) || ' ' || pg_get_constraintdef(c.oid)
-            as fkey_def
-        from pg_constraint c, pg_namespace n1, pg_class t1, pg_namespace n2, pg_class t2
-        where c.contype = 'f' and (c.conrelid = tbl_oid or c.confrelid = tbl_oid)
-            and t1.oid = c.conrelid and n1.oid = t1.relnamespace
-            and t2.oid = c.confrelid and n2.oid = t2.relnamespace
-        order by 1,2,3
-    loop
-        return next fkey;
-    end loop;
-    
-    return;
-end;
-$$ language plpgsql strict stable;
-
-
-create or replace function londiste.find_rel_oid(tbl text, kind text)
-returns oid as $$
-declare
-    res      oid;
-    pos      integer;
-    schema   text;
-    name     text;
-begin
-    pos := position('.' in tbl);
-    if pos > 0 then
-        schema := substring(tbl for pos - 1);
-        name := substring(tbl from pos + 1);
-    else
-        schema := 'public';
-        name := tbl;
-    end if;
-    select c.oid into res
-      from pg_namespace n, pg_class c
-     where c.relnamespace = n.oid
-       and c.relkind = kind
-       and n.nspname = schema and c.relname = name;
-    if not found then
-        if kind = 'r' then
-            raise exception 'table not found';
-        elsif kind = 'S' then
-            raise exception 'seq not found';
-        else
-            raise exception 'weird relkind';
-        end if;
-    end if;
-
-    return res;
-end;
-$$ language plpgsql strict stable;
-
-create or replace function londiste.find_table_oid(tbl text)
-returns oid as $$
-begin
-    return londiste.find_rel_oid(tbl, 'r');
-end;
-$$ language plpgsql strict stable;
-
-create or replace function londiste.find_seq_oid(tbl text)
-returns oid as $$
-begin
-    return londiste.find_rel_oid(tbl, 'S');
-end;
-$$ language plpgsql strict stable;
-
-
-create or replace function londiste.find_table_triggers(i_table_name text)
-returns setof londiste.subscriber_pending_triggers as $$
-declare
-    tg        record;
-begin
-    for tg in
-        select n.nspname || '.' || c.relname as table_name, t.tgname::text as name, pg_get_triggerdef(t.oid) as def 
-        from pg_trigger t, pg_class c, pg_namespace n
-        where n.oid = c.relnamespace and c.oid = t.tgrelid
-            and t.tgrelid = londiste.find_table_oid(i_table_name)
-            and not t.tgisconstraint
-    loop
-        return next tg;
-    end loop;
-    
-    return;
-end;
-$$ language plpgsql strict stable;
-
-create or replace function londiste.get_last_tick(i_consumer text)
-returns bigint as $$
-declare
-    res   bigint;
-begin
-    select last_tick_id into res
-      from londiste.completed
-     where consumer_id = i_consumer;
-    return res;
-end;
-$$ language plpgsql security definer strict stable;
-
 
 create or replace function londiste.link_source(i_dst_name text)
 returns text as $$
@@ -164,7 +28,7 @@ declare
     tmp   record;
     list2 text;
 begin
-    sql := 'select ' || a_field || ' as name from ' || a_table
+    sql := 'select ' || quote_ident(a_field) || ' as name from ' || londiste.quote_fqname(a_table)
         || ' where queue_name = ' || quote_literal(a_queue)
         || ' order by 1';
     list2 := '';
@@ -247,6 +111,23 @@ end;
 $$ language plpgsql security definer;
 
 
+create or replace function londiste.provider_get_seq_list(i_queue_name text)
+returns setof text as $$
+declare
+    rec record;
+begin
+    for rec in
+        select seq_name from londiste.provider_seq
+            where queue_name = i_queue_name
+            order by nr
+    loop
+        return next rec.seq_name;
+    end loop;
+    return;
+end;
+$$ language plpgsql security definer;
+
+
 create or replace function londiste.provider_add_seq(
     i_queue_name text, i_seq_name text)
 returns integer as $$
@@ -267,222 +148,95 @@ begin
 
     insert into londiste.provider_seq (queue_name, seq_name)
         values (i_queue_name, i_seq_name);
-    perform londiste.provider_notify_change(i_queue_name);
 
     return 0;
 end;
 $$ language plpgsql security definer;
 
-create or replace function londiste.provider_add_table(
-    i_queue_name    text,
-    i_table_name    text,
-    i_col_types     text
-) returns integer strict as $$
-declare
-    tgname text;
-    sql    text;
-begin
-    if londiste.link_source(i_queue_name) is not null then
-        raise exception 'Linked queue, manipulation not allowed';
-    end if;
 
-    if position('k' in i_col_types) < 1 then
-        raise exception 'need key column';
-    end if;
-    if position('.' in i_table_name) < 1 then
-        raise exception 'need fully-qualified table name';
-    end if;
-    select queue_name into tgname
-        from pgq.queue where queue_name = i_queue_name;
-    if not found then
-        raise exception 'no such event queue';
-    end if;
-
-    tgname := i_queue_name || '_logger';
-    tgname := replace(lower(tgname), '.', '_');
-    insert into londiste.provider_table
-        (queue_name, table_name, trigger_name)
-        values (i_queue_name, i_table_name, tgname);
-
-    perform londiste.provider_create_trigger(
-        i_queue_name, i_table_name, i_col_types);
-
-    return 1;
-end;
-$$ language plpgsql security definer;
-
-create or replace function londiste.provider_add_table(
-    i_queue_name text,
-    i_table_name text
-) returns integer as $$
-begin
-    return londiste.provider_add_table(i_queue_name, i_table_name,
-        londiste.find_column_types(i_table_name));
-end;
-$$ language plpgsql security definer;
-
-
-create or replace function londiste.provider_create_trigger(
-    i_queue_name    text,
-    i_table_name    text,
-    i_col_types     text
-) returns integer strict as $$
-declare
-    tgname text;
-begin
-    select trigger_name into tgname
-        from londiste.provider_table
-        where queue_name = i_queue_name
-          and table_name = i_table_name;
-    if not found then
-        raise exception 'table not found';
-    end if;
-
-    execute 'create trigger ' || quote_ident(tgname)
-        || ' after insert or update or delete on '
-        || londiste.quote_fqname(i_table_name)
-        || ' for each row execute procedure pgq.logtriga('
-        || quote_literal(i_queue_name) || ', '
-        || quote_literal(i_col_types) || ', '
-        || quote_literal(i_table_name) || ')';
-
-    return 1;
-end;
-$$ language plpgsql security definer;
-
-
-create or replace function londiste.provider_get_seq_list(i_queue_name text)
-returns setof text as $$
-declare
-    rec record;
-begin
-    for rec in
-        select seq_name from londiste.provider_seq
-            where queue_name = i_queue_name
-            order by nr
-    loop
-        return next rec.seq_name;
-    end loop;
-    return;
-end;
-$$ language plpgsql security definer;
-
-
-create or replace function londiste.provider_get_table_list(i_queue text)
-returns setof londiste.ret_provider_table_list as $$ 
-declare 
-    rec   londiste.ret_provider_table_list%rowtype; 
-begin 
-    for rec in 
-        select table_name, trigger_name
-            from londiste.provider_table
-            where queue_name = i_queue
-            order by nr
-    loop
-        return next rec;
-    end loop; 
-    return;
-end; 
-$$ language plpgsql security definer;
-
-
-create or replace function londiste.provider_notify_change(i_queue_name text)
-returns integer as $$
-declare
-    res      text;
-    tbl      record;
-begin
-    res := '';
-    for tbl in
-        select table_name from londiste.provider_table
-            where queue_name = i_queue_name
-            order by nr
-    loop
-        if res = '' then
-            res := tbl.table_name;
-        else
-            res := res || ',' || tbl.table_name;
-        end if;
-    end loop;
-    
-    perform pgq.insert_event(i_queue_name, 'T', res);
-
-    return 1;
-end;
-$$ language plpgsql security definer;
-
-
-create or replace function londiste.provider_refresh_trigger(
-    i_queue_name    text,
-    i_table_name    text,
-    i_col_types     text
-) returns integer strict as $$
-declare
-    t_name   text;
-    tbl_oid  oid;
-begin
-    select trigger_name into t_name
-        from londiste.provider_table
-        where queue_name = i_queue_name
-          and table_name = i_table_name;
-    if not found then
-        raise exception 'table not found';
-    end if;
-
-    tbl_oid := londiste.find_table_oid(i_table_name);
-    perform 1 from pg_trigger
-        where tgrelid = tbl_oid
-          and tgname = t_name;
-    if found then
-        execute 'drop trigger ' || quote_ident(t_name)
-            || ' on ' || londiste.quote_fqname(i_table_name);
-    end if;
-
-    perform londiste.provider_create_trigger(i_queue_name, i_table_name, i_col_types);
-
-    return 1;
-end;
-$$ language plpgsql security definer;
-
-create or replace function londiste.provider_refresh_trigger(
-    i_queue_name    text,
-    i_table_name    text
-) returns integer strict as $$
-begin
-    return londiste.provider_refresh_trigger(i_queue_name, i_table_name,
-                            londiste.find_column_types(i_table_name));
-end;
-$$ language plpgsql security definer;
-
-
-
-
-create or replace function londiste.provider_remove_seq(
+create or replace function londiste.subscriber_remove_seq(
     i_queue_name text, i_seq_name text)
 returns integer as $$
 declare
     link text;
 begin
-    -- check if linked queue
-    link := londiste.link_source(i_queue_name);
-    if link is not null then
-        raise exception 'Linked queue, cannot modify';
-    end if;
-
-    delete from londiste.provider_seq
+    delete from londiste.subscriber_seq
         where queue_name = i_queue_name
           and seq_name = i_seq_name;
     if not found then
-        raise exception 'seq not attached';
+        raise exception 'no such seq?';
     end if;
 
-    perform londiste.provider_notify_change(i_queue_name);
+    -- update linked queue if needed
+    link := londiste.link_dest(i_queue_name);
+    if link is not null then
+        delete from londiste.provider_seq
+         where queue_name = link
+           and seq_name = i_seq_name;
+        perform londiste.provider_notify_change(link);
+    end if;
 
     return 0;
 end;
 $$ language plpgsql security definer;
 
 
+create or replace function londiste.quote_fqname(i_name text)
+returns text as $$
+declare
+    res     text;
+    pos     integer;
+    s       text;
+    n       text;
+begin
+    pos := position('.' in i_name);
+    if pos > 0 then
+        s := substring(i_name for pos - 1);
+        n := substring(i_name from pos + 1);
+    else
+        s := 'public';
+        n := i_name;
+    end if;
+    return quote_ident(s) || '.' || quote_ident(n);
+end;
+$$ language plpgsql strict immutable;
+
+
+create or replace function londiste.find_table_triggers(i_table_name text)
+returns setof londiste.subscriber_pending_triggers as $$
+declare
+    tg        record;
+    ver       int4;
+begin
+    select setting::int4 into ver from pg_settings
+     where name = 'server_version_num';
+
+    if ver >= 90000 then
+        for tg in
+            select n.nspname || '.' || c.relname as table_name, t.tgname::text as name, pg_get_triggerdef(t.oid) as def 
+            from pg_trigger t, pg_class c, pg_namespace n
+            where n.oid = c.relnamespace and c.oid = t.tgrelid
+                and t.tgrelid = londiste.find_table_oid(i_table_name)
+                and not t.tgisinternal
+        loop
+            return next tg;
+        end loop;
+    else
+        for tg in
+            select n.nspname || '.' || c.relname as table_name, t.tgname::text as name, pg_get_triggerdef(t.oid) as def 
+            from pg_trigger t, pg_class c, pg_namespace n
+            where n.oid = c.relnamespace and c.oid = t.tgrelid
+                and t.tgrelid = londiste.find_table_oid(i_table_name)
+                and not t.tgisconstraint
+        loop
+            return next tg;
+        end loop;
+    end if;
+    
+    return;
+end;
+$$ language plpgsql strict stable;
+
 create or replace function londiste.provider_remove_table(
     i_queue_name   text,
     i_table_name   text
@@ -519,26 +273,31 @@ end;
 $$ language plpgsql security definer;
 
 
-
-create or replace function londiste.quote_fqname(i_name text)
+create or replace function londiste.find_column_types(tbl text)
 returns text as $$
 declare
-    res     text;
-    pos     integer;
-    s       text;
-    n       text;
+    res      text;
+    col      record;
+    tbl_oid  oid;
 begin
-    pos := position('.' in i_name);
-    if pos > 0 then
-        s := substring(i_name for pos - 1);
-        n := substring(i_name from pos + 1);
-    else
-        s := 'public';
-        n := i_name;
-    end if;
-    return quote_ident(s) || '.' || quote_ident(n);
+    tbl_oid := londiste.find_table_oid(tbl);
+    res := '';
+    for col in 
+        SELECT CASE WHEN k.attname IS NOT NULL THEN 'k' ELSE 'v' END AS type
+            FROM pg_attribute a LEFT JOIN (
+                SELECT k.attname FROM pg_index i, pg_attribute k
+                 WHERE i.indrelid = tbl_oid AND k.attrelid = i.indexrelid
+                   AND i.indisprimary AND k.attnum > 0 AND NOT k.attisdropped
+                ) k ON (k.attname = a.attname)
+            WHERE a.attrelid = tbl_oid AND a.attnum > 0 AND NOT a.attisdropped
+            ORDER BY a.attnum
+    loop
+        res := res || col.type;
+    end loop;
+
+    return res;
 end;
-$$ language plpgsql strict immutable;
+$$ language plpgsql strict stable;
 
 
 create or replace function londiste.set_last_tick(
@@ -587,20 +346,6 @@ end;
 $$ language plpgsql security definer;
 
 
-create or replace function londiste.subscriber_add_table(
-    i_queue_name text, i_table text)
-returns integer as $$
-begin
-    insert into londiste.subscriber_table (queue_name, table_name)
-        values (i_queue_name, i_table);
-
-    -- linked queue is updated, when the table is copied
-
-    return 0;
-end;
-$$ language plpgsql security definer;
-
-
 
 create or replace function londiste.subscriber_get_table_pending_fkeys(i_table_name text) 
 returns setof londiste.subscriber_pending_fkeys as $$
@@ -688,129 +433,137 @@ begin
 end;
 $$ language plpgsql;
 
-create or replace function londiste.subscriber_get_seq_list(i_queue_name text)
-returns setof text as $$
+create or replace function londiste.provider_refresh_trigger(
+    i_queue_name    text,
+    i_table_name    text,
+    i_col_types     text
+) returns integer strict as $$
 declare
-    rec record;
+    t_name   text;
+    tbl_oid  oid;
 begin
-    for rec in
-        select seq_name from londiste.subscriber_seq
-            where queue_name = i_queue_name
-            order by nr
-    loop
-        return next rec.seq_name;
-    end loop;
-    return;
+    select trigger_name into t_name
+        from londiste.provider_table
+        where queue_name = i_queue_name
+          and table_name = i_table_name;
+    if not found then
+        raise exception 'table not found';
+    end if;
+
+    tbl_oid := londiste.find_table_oid(i_table_name);
+    perform 1 from pg_trigger
+        where tgrelid = tbl_oid
+          and tgname = t_name;
+    if found then
+        execute 'drop trigger ' || quote_ident(t_name)
+            || ' on ' || londiste.quote_fqname(i_table_name);
+    end if;
+
+    perform londiste.provider_create_trigger(i_queue_name, i_table_name, i_col_types);
+
+    return 1;
+end;
+$$ language plpgsql security definer;
+
+create or replace function londiste.provider_refresh_trigger(
+    i_queue_name    text,
+    i_table_name    text
+) returns integer strict as $$
+begin
+    return londiste.provider_refresh_trigger(i_queue_name, i_table_name,
+                            londiste.find_column_types(i_table_name));
 end;
 $$ language plpgsql security definer;
 
 
-create or replace function londiste.subscriber_get_table_list(i_queue_name text)
-returns setof londiste.ret_subscriber_table as $$
-declare
-    rec londiste.ret_subscriber_table%rowtype;
-begin
-    for rec in
-        select table_name, merge_state, snapshot, trigger_name, skip_truncate
-          from londiste.subscriber_table
-         where queue_name = i_queue_name
-         order by nr
-    loop
-        return next rec;
-    end loop;
-    return;
-end;
-$$ language plpgsql security definer;
-
--- compat
-create or replace function londiste.get_table_state(i_queue text)
-returns setof londiste.subscriber_table as $$
-declare
-    rec londiste.subscriber_table%rowtype;
-begin
-    for rec in
-        select * from londiste.subscriber_table
-            where queue_name = i_queue
-            order by nr
-    loop
-        return next rec;
-    end loop;
-    return;
-end;
-$$ language plpgsql security definer;
 
 
-create or replace function londiste.subscriber_remove_seq(
+create or replace function londiste.provider_remove_seq(
     i_queue_name text, i_seq_name text)
 returns integer as $$
 declare
     link text;
 begin
-    delete from londiste.subscriber_seq
+    -- check if linked queue
+    link := londiste.link_source(i_queue_name);
+    if link is not null then
+        raise exception 'Linked queue, cannot modify';
+    end if;
+
+    delete from londiste.provider_seq
         where queue_name = i_queue_name
           and seq_name = i_seq_name;
     if not found then
-        raise exception 'no such seq?';
+        raise exception 'seq not attached';
     end if;
 
-    -- update linked queue if needed
-    link := londiste.link_dest(i_queue_name);
-    if link is not null then
-        delete from londiste.provider_seq
-         where queue_name = link
-           and seq_name = i_seq_name;
-        perform londiste.provider_notify_change(link);
-    end if;
+    perform londiste.provider_notify_change(i_queue_name);
 
     return 0;
 end;
 $$ language plpgsql security definer;
 
 
-create or replace function londiste.subscriber_remove_table(
-    i_queue_name text, i_table text)
-returns integer as $$
+create or replace function londiste.get_last_tick(i_consumer text)
+returns bigint as $$
 declare
-    link  text;
+    res   bigint;
 begin
-    delete from londiste.subscriber_table
-     where queue_name = i_queue_name
-       and table_name = i_table;
-    if not found then
-        raise exception 'no such table';
-    end if;
-
-    -- sync link
-    link := londiste.link_dest(i_queue_name);
-    if link is not null then
-        delete from londiste.provider_table
-            where queue_name = link
-              and table_name = i_table;
-        perform londiste.provider_notify_change(link);
-    end if;
-
-    return 0;
+    select last_tick_id into res
+      from londiste.completed
+     where consumer_id = i_consumer;
+    return res;
 end;
-$$ language plpgsql security definer;
+$$ language plpgsql security definer strict stable;
 
-
-create or replace function londiste.subscriber_set_skip_truncate(
-    i_queue text,
-    i_table text,
-    i_value bool)
-returns integer as $$
+create or replace function londiste.find_rel_oid(tbl text, kind text)
+returns oid as $$
+declare
+    res      oid;
+    pos      integer;
+    schema   text;
+    name     text;
 begin
-    update londiste.subscriber_table
-       set skip_truncate = i_value
-     where queue_name = i_queue
-       and table_name = i_table;
+    pos := position('.' in tbl);
+    if pos > 0 then
+        schema := substring(tbl for pos - 1);
+        name := substring(tbl from pos + 1);
+    else
+        schema := 'public';
+        name := tbl;
+    end if;
+    select c.oid into res
+      from pg_namespace n, pg_class c
+     where c.relnamespace = n.oid
+       and c.relkind = kind
+       and n.nspname = schema and c.relname = name;
     if not found then
-        raise exception 'table not found';
+        if kind = 'r' then
+            raise exception 'table not found';
+        elsif kind = 'S' then
+            raise exception 'seq not found';
+        else
+            raise exception 'weird relkind';
+        end if;
     end if;
 
-    return 1;
+    return res;
 end;
-$$ language plpgsql security definer;
+$$ language plpgsql strict stable;
+
+create or replace function londiste.find_table_oid(tbl text)
+returns oid as $$
+begin
+    return londiste.find_rel_oid(tbl, 'r');
+end;
+$$ language plpgsql strict stable;
+
+create or replace function londiste.find_seq_oid(tbl text)
+returns oid as $$
+begin
+    return londiste.find_rel_oid(tbl, 'S');
+end;
+$$ language plpgsql strict stable;
 
 
 create or replace function londiste.subscriber_set_table_state(
@@ -876,6 +629,97 @@ $$ language plpgsql security definer;
 
 
 
+create or replace function londiste.find_table_fkeys(i_table_name text)
+returns setof londiste.subscriber_pending_fkeys as $$
+declare
+    fkey      record;
+    tbl_oid   oid;
+begin
+    select londiste.find_table_oid(i_table_name) into tbl_oid;
+        
+    for fkey in
+        select n1.nspname || '.' || t1.relname as from_table, n2.nspname || '.' || t2.relname as to_table,
+            conname::text as fkey_name, 
+            'alter table only ' || quote_ident(n1.nspname) || '.' || quote_ident(t1.relname)
+            || ' add constraint ' || quote_ident(conname::text) || ' ' || pg_get_constraintdef(c.oid)
+            as fkey_def
+        from pg_constraint c, pg_namespace n1, pg_class t1, pg_namespace n2, pg_class t2
+        where c.contype = 'f' and (c.conrelid = tbl_oid or c.confrelid = tbl_oid)
+            and t1.oid = c.conrelid and n1.oid = t1.relnamespace
+            and t2.oid = c.confrelid and n2.oid = t2.relnamespace
+        order by 1,2,3
+    loop
+        return next fkey;
+    end loop;
+    
+    return;
+end;
+$$ language plpgsql strict stable;
+
+
+create or replace function londiste.provider_add_table(
+    i_queue_name    text,
+    i_table_name    text,
+    i_col_types     text
+) returns integer strict as $$
+declare
+    tgname text;
+    sql    text;
+begin
+    if londiste.link_source(i_queue_name) is not null then
+        raise exception 'Linked queue, manipulation not allowed';
+    end if;
+
+    if position('k' in i_col_types) < 1 then
+        raise exception 'need key column';
+    end if;
+    if position('.' in i_table_name) < 1 then
+        raise exception 'need fully-qualified table name';
+    end if;
+    select queue_name into tgname
+        from pgq.queue where queue_name = i_queue_name;
+    if not found then
+        raise exception 'no such event queue';
+    end if;
+
+    tgname := i_queue_name || '_logger';
+    tgname := replace(lower(tgname), '.', '_');
+    insert into londiste.provider_table
+        (queue_name, table_name, trigger_name)
+        values (i_queue_name, i_table_name, tgname);
+
+    perform londiste.provider_create_trigger(
+        i_queue_name, i_table_name, i_col_types);
+
+    return 1;
+end;
+$$ language plpgsql security definer;
+
+create or replace function londiste.provider_add_table(
+    i_queue_name text,
+    i_table_name text
+) returns integer as $$
+begin
+    return londiste.provider_add_table(i_queue_name, i_table_name,
+        londiste.find_column_types(i_table_name));
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.subscriber_add_table(
+    i_queue_name text, i_table text)
+returns integer as $$
+begin
+    insert into londiste.subscriber_table (queue_name, table_name)
+        values (i_queue_name, i_table);
+
+    -- linked queue is updated, when the table is copied
+
+    return 0;
+end;
+$$ language plpgsql security definer;
+
+
 create or replace function londiste.subscriber_get_table_pending_triggers(i_table_name text)
 returns setof londiste.subscriber_pending_triggers as $$
 declare
@@ -976,10 +820,181 @@ $$ language plpgsql;
 
 
 
+create or replace function londiste.subscriber_get_table_list(i_queue_name text)
+returns setof londiste.ret_subscriber_table as $$
+declare
+    rec londiste.ret_subscriber_table%rowtype;
+begin
+    for rec in
+        select table_name, merge_state, snapshot, trigger_name, skip_truncate
+          from londiste.subscriber_table
+         where queue_name = i_queue_name
+         order by nr
+    loop
+        return next rec;
+    end loop;
+    return;
+end;
+$$ language plpgsql security definer;
+
+-- compat
+create or replace function londiste.get_table_state(i_queue text)
+returns setof londiste.subscriber_table as $$
+declare
+    rec londiste.subscriber_table%rowtype;
+begin
+    for rec in
+        select * from londiste.subscriber_table
+            where queue_name = i_queue
+            order by nr
+    loop
+        return next rec;
+    end loop;
+    return;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.subscriber_set_skip_truncate(
+    i_queue text,
+    i_table text,
+    i_value bool)
+returns integer as $$
+begin
+    update londiste.subscriber_table
+       set skip_truncate = i_value
+     where queue_name = i_queue
+       and table_name = i_table;
+    if not found then
+        raise exception 'table not found';
+    end if;
+
+    return 1;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.provider_notify_change(i_queue_name text)
+returns integer as $$
+declare
+    res      text;
+    tbl      record;
+begin
+    res := '';
+    for tbl in
+        select table_name from londiste.provider_table
+            where queue_name = i_queue_name
+            order by nr
+    loop
+        if res = '' then
+            res := tbl.table_name;
+        else
+            res := res || ',' || tbl.table_name;
+        end if;
+    end loop;
+    
+    perform pgq.insert_event(i_queue_name, 'T', res);
+
+    return 1;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.subscriber_get_seq_list(i_queue_name text)
+returns setof text as $$
+declare
+    rec record;
+begin
+    for rec in
+        select seq_name from londiste.subscriber_seq
+            where queue_name = i_queue_name
+            order by nr
+    loop
+        return next rec.seq_name;
+    end loop;
+    return;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.provider_create_trigger(
+    i_queue_name    text,
+    i_table_name    text,
+    i_col_types     text
+) returns integer strict as $$
+declare
+    tgname text;
+begin
+    select trigger_name into tgname
+        from londiste.provider_table
+        where queue_name = i_queue_name
+          and table_name = i_table_name;
+    if not found then
+        raise exception 'table not found';
+    end if;
+
+    execute 'create trigger ' || quote_ident(tgname)
+        || ' after insert or update or delete on '
+        || londiste.quote_fqname(i_table_name)
+        || ' for each row execute procedure pgq.logtriga('
+        || quote_literal(i_queue_name) || ', '
+        || quote_literal(i_col_types) || ', '
+        || quote_literal(i_table_name) || ')';
+
+    return 1;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.subscriber_remove_table(
+    i_queue_name text, i_table text)
+returns integer as $$
+declare
+    link  text;
+begin
+    delete from londiste.subscriber_table
+     where queue_name = i_queue_name
+       and table_name = i_table;
+    if not found then
+        raise exception 'no such table';
+    end if;
+
+    -- sync link
+    link := londiste.link_dest(i_queue_name);
+    if link is not null then
+        delete from londiste.provider_table
+            where queue_name = link
+              and table_name = i_table;
+        perform londiste.provider_notify_change(link);
+    end if;
+
+    return 0;
+end;
+$$ language plpgsql security definer;
+
+
+create or replace function londiste.provider_get_table_list(i_queue text)
+returns setof londiste.ret_provider_table_list as $$ 
+declare 
+    rec   londiste.ret_provider_table_list%rowtype; 
+begin 
+    for rec in 
+        select table_name, trigger_name
+            from londiste.provider_table
+            where queue_name = i_queue
+            order by nr
+    loop
+        return next rec;
+    end loop; 
+    return;
+end; 
+$$ language plpgsql security definer;
+
+
 create or replace function londiste.version()
 returns text as $$
 begin
-    return '2.1.7';
+    return '2.1.12';
 end;
 $$ language plpgsql;
 
