@@ -724,6 +724,10 @@ declare
     rec    record;
 begin
     cnt := 0;
+
+    -- allow only single event mover at a time, without affecting inserts
+    lock table pgq.retry_queue in share update exclusive mode;
+
     for rec in
         select queue_name,
                ev_id, ev_time, ev_owner, ev_retry, ev_type, ev_data,
@@ -761,13 +765,15 @@ returns integer as $$
 --      i_queue_name        - Name of the queue
 --
 -- Returns:
---      nothing
+--      1 if rotation happened, otherwise 0.
 -- ----------------------------------------------------------------------
 declare
-    badcnt  integer;
-    cf      record;
-    nr      integer;
-    tbl     text;
+    badcnt          integer;
+    cf              record;
+    nr              integer;
+    tbl             text;
+    lowest_tick_id  int8;
+    lowest_xmin     int8;
 begin
     -- check if needed and load record
     select * from pgq.queue into cf
@@ -780,22 +786,26 @@ begin
         return 0;
     end if;
 
-    -- check if any consumer is on previous table
-    select coalesce(count(*), 0) into badcnt
-        from pgq.subscription, pgq.tick
-        where txid_snapshot_xmin(tick_snapshot) < cf.queue_switch_step2
-          and sub_queue = cf.queue_id
-          and tick_queue = cf.queue_id
-          and tick_id = (select tick_id from pgq.tick
-                           where tick_id < sub_last_tick
-                             and tick_queue = sub_queue
-                           order by tick_queue desc, tick_id desc
-                           limit 1);
-    if badcnt > 0 then
-        return 0;
+    -- find lowest tick for that queue
+    select min(sub_last_tick) into lowest_tick_id
+      from pgq.subscription
+     where sub_queue = cf.queue_id;
+
+    -- if some consumer exists
+    if lowest_tick_id is not null then
+        -- is the slowest one still on previous table?
+        select txid_snapshot_xmin(tick_snapshot) into lowest_xmin
+          from pgq.tick
+         where tick_queue = cf.queue_id
+           and tick_id = lowest_tick_id;
+        if lowest_xmin <= cf.queue_switch_step2 then
+            return 0; -- skip rotation then
+        end if;
     end if;
 
-    -- all is fine, calc next table number
+    -- nobody on previous table, we can rotate
+    
+    -- calc next table number and name
     nr := cf.queue_cur_table + 1;
     if nr = cf.queue_ntables then
         nr := 0;
@@ -809,7 +819,7 @@ begin
         execute 'truncate ' || tbl;
     exception
         when lock_not_available then
-            raise warning 'truncate of % failed, skipping rotate', tbl;
+            -- cannot truncate, skipping rotate
             return 0;
     end;
 
@@ -821,7 +831,14 @@ begin
             queue_switch_step2 = NULL
         where queue_id = cf.queue_id;
 
-    -- clean ticks - avoid partial batches
+    -- Clean ticks by using step2 txid from previous rotation.
+    -- That should keep all ticks for all batches that are completely
+    -- in old table.  This keeps them for longer than needed, but:
+    -- 1. we want the pgq.tick table to be big, to avoid Postgres
+    --    accitentally switching to seqscans on that.
+    -- 2. that way we guarantee to consumers that they an be moved
+    --    back on the queue at least for one rotation_period.
+    --    (may help in disaster recovery)
     delete from pgq.tick
         where tick_queue = cf.queue_id
           and txid_snapshot_xmin(tick_snapshot) < cf.queue_switch_step2;
@@ -830,16 +847,15 @@ begin
 end;
 $$ language plpgsql; -- need admin access
 
+
+create or replace function pgq.maint_rotate_tables_step2()
+returns integer as $$
 -- ----------------------------------------------------------------------
 -- Function: pgq.maint_rotate_tables_step2(0)
 --
---      It tag rotation as finished where needed.  It should be
+--      Stores the txid when the rotation was visible.  It should be
 --      called in separate transaction than pgq.maint_rotate_tables_step1()
 -- ----------------------------------------------------------------------
-create or replace function pgq.maint_rotate_tables_step2()
-returns integer as $$
--- visibility tracking.  this should run in separate
--- tranaction than step1
 begin
     update pgq.queue
        set queue_switch_step2 = txid_current()
@@ -1833,7 +1849,7 @@ returns text as $$
 --      that is only bumped when PGQ database code changes.
 -- ----------------------------------------------------------------------
 begin
-    return '2.1.7';
+    return '2.1.8';
 end;
 $$ language plpgsql;
 
@@ -2096,9 +2112,9 @@ $$ language plpgsql security definer;
 -- ----------------------------------------------------------------------
 -- Function: pgq.logtriga()
 --
---      Non-automatic SQL trigger.  It puts row data in partial SQL form into
---      queue.  It does not auto-detect table structure, it needs to be passed
---      as trigger arg.
+--      Deprecated - non-automatic SQL trigger.  It puts row data in partial
+--      SQL form into queue.  It does not auto-detect table structure,
+--      it needs to be passed as trigger arg.
 --
 -- Purpose:
 --      Used by Londiste to generate replication events.  The "partial SQL"
@@ -2142,11 +2158,13 @@ AS '$libdir/pgq_triggers', 'pgq_logtriga' LANGUAGE C;
 --      SKIP                - The actual operation should be skipped
 --      ignore=col1[,col2]  - don't look at the specified arguments
 --      pkey=col1[,col2]    - Set pkey fields for the table, autodetection will be skipped
+--      backup              - Put urlencoded contents of old row to ev_extra2
 --
 -- Queue event fields:
 --    ev_type     - I/U/D
 --    ev_data     - partial SQL statement
 --    ev_extra1   - table name
+--    ev_extra2   - optional urlencoded backup
 --
 -- ----------------------------------------------------------------------
 CREATE OR REPLACE FUNCTION pgq.sqltriga() RETURNS trigger
@@ -2170,11 +2188,13 @@ AS '$libdir/pgq_triggers', 'pgq_sqltriga' LANGUAGE C;
 --      SKIP                - The actual operation should be skipped
 --      ignore=col1[,col2]  - don't look at the specified arguments
 --      pkey=col1[,col2]    - Set pkey fields for the table, autodetection will be skipped
+--      backup              - Put urlencoded contents of old row to ev_extra2
 --
 -- Queue event fields:
 --      ev_type      - I/U/D ':' pkey_column_list
 --      ev_data      - column values urlencoded
 --      ev_extra1    - table name
+--      ev_extra2    - optional urlencoded backup
 --
 -- Regular listen trigger example:
 -- >   CREATE TRIGGER triga_nimi AFTER INSERT OR UPDATE ON customer
